@@ -1,10 +1,9 @@
 package Hailo::Role::Engine;
+our $VERSION = '0.31';
 use 5.10.0;
-use MooseX::Role::Strict;
+use Any::Moose '::Role';
 use namespace::clean -except => 'meta';
 
-our $VERSION = '0.09';
-
 requires 'learn';
 requires 'reply';
 
@@ -26,11 +25,6 @@ A C<Hailo::Engine::*> should implement the following attributes:
 
 A L<storage|Hailo::Role::Storage> object the engine should use to get data from.
 
-=head2 C<tokenizer>
-
-A L<tokenizer|Hailo::Role::Tokenizer> object the engine should use to
-tokenize data with.
-
 =head1 METHODS
 
 =head2 C<new>
